#include <stdio.h>
#include <windows.h>

#define MENU_SEPARATOR_POS 5
#define MENU_SEPARATOR_ID 0xDC00
#define MENU_REMEMBER_ID 0xDC10
#define MENU_REMEMBER_POS 6
#define MENU_REMEMBER_CAPTION "Remember Size/Pos"

#define INFO_FORMAT "x%dy%dw%dh%d"
#define MAX_INFO_SIZE 21

char filePath[MAX_PATH];
char iniPath[MAX_PATH];

void WriteData(HWND hwnd, BOOL save) {
    char wndClass[MAX_PATH];
    char value[MAX_INFO_SIZE];
    RECT rect;
    GetWindowRect(hwnd, &rect);
    GetClassName(hwnd, wndClass, MAX_PATH);

    sprintf(value, INFO_FORMAT, (int)rect.left, (int)rect.top, 
        (int)(rect.right - rect.left), (int)(rect.bottom - rect.top));

    WritePrivateProfileString(filePath, save ? wndClass : NULL, value, iniPath);
}

BOOL ReadData(HWND hwnd, int *iX, int *iY, int *iW, int *iH) {
    char value[MAX_INFO_SIZE];
    char wndClass[MAX_PATH];

    GetClassName(hwnd, wndClass, MAX_PATH);

    if(GetPrivateProfileString(filePath, wndClass, "", value, MAX_INFO_SIZE, iniPath) && value[0])
        return sscanf(value, INFO_FORMAT, iX, iY, iW, iH) == 4;

    return FALSE;
}

__declspec(dllexport) LRESULT CALLBACK CBTProc(int nCode, WPARAM wParam, LPARAM lParam) {
    if(nCode == HCBT_CREATEWND) {
        CBT_CREATEWND *cbt = (CBT_CREATEWND*)lParam;
        HMENU hm = GetSystemMenu((HWND)wParam, FALSE);

        if(hm != NULL) {
            //Read saved pos and size if any
            UINT checked = ReadData((HWND)wParam, &cbt->lpcs->x, &cbt->lpcs->y, 
                &cbt->lpcs->cx, &cbt->lpcs->cy) ? MF_CHECKED : MF_UNCHECKED;

            //Insert the REMEMBER_MENU
            InsertMenu(hm, MENU_SEPARATOR_POS, MF_BYPOSITION | MF_SEPARATOR, MENU_SEPARATOR_ID, NULL);
            InsertMenu(hm, MENU_REMEMBER_POS, MF_BYPOSITION | checked, MENU_REMEMBER_ID, MENU_REMEMBER_CAPTION);
        }
    } else if(nCode == HCBT_DESTROYWND) {
        HMENU hm = GetSystemMenu((HWND)wParam, FALSE);

        if(hm != NULL)
            //Save or clear the pos and size
            WriteData((HWND)wParam, GetMenuState(hm, MENU_REMEMBER_ID, MF_BYCOMMAND) & MF_CHECKED);
    }

    return CallNextHookEx(NULL, nCode, wParam, lParam);
}

__declspec(dllexport) LRESULT CALLBACK GetMsgProc(int nCode, WPARAM wParam, LPARAM lParam) {
    MSG *msg = (MSG*)lParam;
    //Handle REMEMBER_MENU
    if(msg->message == WM_SYSCOMMAND && msg->wParam == MENU_REMEMBER_ID) {
        HMENU hm = GetSystemMenu(msg->hwnd, FALSE);
        //Toggle checked state
        UINT checked = (GetMenuState(hm, MENU_REMEMBER_ID, MF_BYCOMMAND) & MF_CHECKED) ^ MF_CHECKED;

        ModifyMenu(hm, MENU_REMEMBER_ID, MF_BYCOMMAND | checked, MENU_REMEMBER_ID, MENU_REMEMBER_CAPTION);
    }

    return CallNextHookEx(NULL, nCode, wParam, lParam);
}

BOOL APIENTRY DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
    if(fdwReason == DLL_PROCESS_ATTACH) {
        //Get the hooked executable filename
        GetModuleFileName(NULL, filePath, MAX_PATH);
        //Get the dll filename
        int size = GetModuleFileName(hinstDLL, iniPath, MAX_PATH);
        //and change the extension to ini
        lstrcpy(iniPath + size - 3, "ini");
    }

    return TRUE;
}
