#include <windows.h>

#define WINDOW_CLASS "DC-RSP"
#define SHELL32_ICON 98
#define NOTIFY_ICON_ID 0xDC
#define NOTIFY_ICON_TIP "Double click to close"

void tearUp(HHOOK *cbtHook, HHOOK *wndHook) {
    char dllPath[MAX_PATH];
    //Get the executable filename
    DWORD size = GetModuleFileName(NULL, dllPath, MAX_PATH);
    //and change the extension to dll
    lstrcpy(dllPath + size - 3, "dll");

    //Try to load the dll
    HINSTANCE hDll = LoadLibrary(dllPath);
    HOOKPROC proc;

    //Quit if any error has happened while loading
    if(hDll == NULL)
        ExitProcess(1);

    //Setup the CBT hook to handle window creation and destruction
    proc = (HOOKPROC)GetProcAddress(hDll, "CBTProc");
    *cbtHook = SetWindowsHookEx(WH_CBT, proc, hDll, 0);

    if(*cbtHook == NULL)
        ExitProcess(2);

    //Setup the Message hook to handle system menu
    proc = (HOOKPROC)GetProcAddress(hDll, "GetMsgProc");
    *wndHook = SetWindowsHookEx(WH_GETMESSAGE, proc, hDll, 0);

    if(*wndHook == NULL)
        ExitProcess(3);
}

inline void tearDown(HHOOK cbtHook, HHOOK wndHook) {
    UnhookWindowsHookEx(cbtHook);
    UnhookWindowsHookEx(wndHook);
}

LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_APP && lParam == WM_LBUTTONDBLCLK)
        PostQuitMessage(0);
    return DefWindowProc(hwnd, uMsg, wParam, lParam);
}

void _start() {
	HINSTANCE hInstance = GetModuleHandle(NULL);

    //Setup windows hook
	HHOOK cbtHook = NULL;
	HHOOK wndHook = NULL;
    tearUp(&cbtHook, &wndHook);

    WNDCLASS wc = {};
    wc.lpfnWndProc   = WindowProc;
    wc.hInstance     = hInstance;
    wc.lpszClassName = WINDOW_CLASS;

    RegisterClass(&wc);

    //Create a hidden window to handle tray icon messages
    HWND hWnd = CreateWindowEx(0, WINDOW_CLASS, NULL, 0, CW_USEDEFAULT, 
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

    //Setup tray notification icon
    NOTIFYICONDATA niData = {};
    niData.cbSize = NOTIFYICONDATA_V1_SIZE;
    niData.uID = NOTIFY_ICON_ID;
    niData.hWnd = hWnd;
    niData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
    niData.uCallbackMessage = WM_APP;
    niData.hIcon = ExtractIcon(hInstance, "shell32.dll", SHELL32_ICON);
    lstrcpy(niData.szTip, NOTIFY_ICON_TIP);

    Shell_NotifyIcon(NIM_ADD, &niData);

    //Message loop
    MSG msg = {};
    while (GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    //Delete tray icon
    Shell_NotifyIcon(NIM_DELETE, &niData);
    DestroyIcon(niData.hIcon);

    //Unhook windows
	tearDown(cbtHook, wndHook);
    
    ExitProcess(0);
}
